@@ -1,3 +1,4 @@
+import os
 import re
 import subprocess
 from typing import Any, Callable, Iterator, List, Optional, Union
@@ -23,35 +24,42 @@ def _probe_duration(cmd: List[str]) -> Optional[int]:
     Returns:
         Optional[int]: The duration in milliseconds.
     """
+    file_names = []
+    for i, arg in enumerate(cmd):
+        if arg == "-i":
+            file_name = cmd[i + 1]
 
-    def _get_file_name(cmd: List[str]) -> Optional[str]:
+            # filter for filenames that we can probe, i.e. regular files
+            if os.path.isfile(file_name):
+                file_names.append(file_name)
+
+    if len(file_names) == 0:
+        return None
+
+    durations = []
+
+    for file_name in file_names:
         try:
-            idx = cmd.index("-i")
-            return cmd[idx + 1]
-        except ValueError:
+            output = subprocess.check_output(
+                [
+                    "ffprobe",
+                    "-loglevel",
+                    "error",
+                    "-hide_banner",
+                    "-show_entries",
+                    "format=duration",
+                    "-of",
+                    "default=noprint_wrappers=1:nokey=1",
+                    file_name,
+                ],
+                universal_newlines=True,
+            )
+            durations.append(int(float(output.strip()) * 1000))
+        except Exception:
+            # TODO: add logging
             return None
 
-    file_name = _get_file_name(cmd)
-    if file_name is None:
-        return None
-
-    try:
-        output = subprocess.check_output(
-            [
-                "ffprobe",
-                "-hide_banner",
-                "-show_entries",
-                "format=duration",
-                "-of",
-                "default=noprint_wrappers=1:nokey=1",
-                file_name,
-            ],
-            universal_newlines=True,
-        )
-        return int(float(output.strip()) * 1000)
-    except Exception:
-        # TODO: add logging
-        return None
+    return max(durations) if "-shortest" not in cmd else min(durations)
 
 
 def _uses_error_loglevel(cmd: List[str]) -> bool:
@@ -65,6 +73,42 @@ def _uses_error_loglevel(cmd: List[str]) -> bool:
         return False
 
 
+def _get_inputs_with_options(cmd: List[str]) -> List[List[str]]:
+    """
+    Collect all inputs with their options.
+    For example, input is:
+
+        ffmpeg -i input1.mp4 -i input2.mp4 -i input3.mp4 -filter_complex ...
+
+    Output is:
+
+        [
+            ["-i", "input1.mp4"],
+            ["-i", "input2.mp4"],
+            ["-i", "input3.mp4"],
+        ]
+
+    Another example:
+
+        ffmpeg -f lavfi -i color=c=black:s=1920x1080 -loop 1 -i image.png -filter_complex ...
+
+    Output is:
+
+        [
+            ["-f", "lavfi", "-i", "color=c=black:s=1920x1080"],
+            ["-loop", "1", "-i", "image.png"],
+        ]
+    """
+    inputs = []
+    prev_index = 0
+    for i, arg in enumerate(cmd):
+        if arg == "-i":
+            inputs.append(cmd[prev_index : i + 2])
+            prev_index = i + 2
+
+    return inputs
+
+
 class FfmpegProgress:
     DUR_REGEX = re.compile(
         r"Duration: (?P<hour>\d{2}):(?P<min>\d{2}):(?P<sec>\d{2})\.(?P<ms>\d{2})"
@@ -111,7 +155,7 @@ class FfmpegProgress:
 
     def run_command_with_progress(
         self, popen_kwargs=None, duration_override: Union[float, None] = None
-    ) -> Iterator[int]:
+    ) -> Iterator[float]:
         """
         Run an ffmpeg command, trying to capture the process output and calculate
         the duration / progress.
@@ -125,7 +169,7 @@ class FfmpegProgress:
             RuntimeError: If the command fails, an exception is raised.
 
         Yields:
-            Iterator[int]: A generator that yields the progress in percent.
+            Iterator[float]: A generator that yields the progress in percent.
         """
         if self.dry_run:
             return self.cmd
@@ -134,10 +178,16 @@ class FfmpegProgress:
         if _uses_error_loglevel(self.cmd):
             total_dur = _probe_duration(self.cmd)
 
+        if duration_override is not None:
+            total_dur = int(duration_override * 1000)
+
         cmd_with_progress = (
             [self.cmd[0]] + ["-progress", "-", "-nostats"] + self.cmd[1:]
         )
 
+        # collect all inputs with their options, e.g.
+        inputs_with_options = _get_inputs_with_options(self.cmd)
+
         stderr = []
         base_popen_kwargs = self.base_popen_kwargs.copy()
         if popen_kwargs is not None:
@@ -150,6 +200,8 @@ class FfmpegProgress:
 
         yield 0
 
+        input_idx = 0
+
         while True:
             if self.process.stdout is None:
                 continue
@@ -168,21 +220,39 @@ class FfmpegProgress:
 
             self.stderr = "\n".join(stderr)
 
-            if total_dur is None:
-                total_dur_match = self.DUR_REGEX.search(stderr_line)
-                if total_dur_match:
-                    total_dur = to_ms(**total_dur_match.groupdict())
-                    continue
-                elif duration_override is not None:
-                    # use the override (should apply in the first loop)
-                    total_dur = int(duration_override * 1000)
-                    continue
+            # assign the total duration if it was found. this can happen multiple times for multiple inputs,
+            # in which case we have to determine the overall duration by taking the min/max (dependent on -shortest being present)
+            if (
+                current_dur_match := self.DUR_REGEX.search(stderr_line)
+            ) and duration_override is None:
+                input_options = inputs_with_options[input_idx]
 
-            if total_dur:
-                progress_time = FfmpegProgress.TIME_REGEX.search(stderr_line)
-                if progress_time:
-                    elapsed_time = to_ms(**progress_time.groupdict())
-                    yield int(elapsed_time / total_dur * 100)
+                current_dur_ms: int = to_ms(**current_dur_match.groupdict())
+                # if the previous line had "image2", it's a single image and we assume a really short intrinsic duration (4ms),
+                # but if it's a loop, we assume infinity
+                if "image2" in stderr[-2] and "-loop 1" in " ".join(input_options):
+                    # infinity
+                    current_dur_ms = 2**64
+                if "-shortest" in self.cmd:
+                    total_dur = (
+                        min(total_dur, current_dur_ms)
+                        if total_dur is not None
+                        else current_dur_ms
+                    )
+                else:
+                    total_dur = (
+                        max(total_dur, current_dur_ms)
+                        if total_dur is not None
+                        else current_dur_ms
+                    )
+
+                input_idx += 1
+
+            if (
+                progress_time := FfmpegProgress.TIME_REGEX.search(stderr_line)
+            ) and total_dur is not None:
+                elapsed_time = to_ms(**progress_time.groupdict())
+                yield min(max(round(elapsed_time / total_dur * 100, 2), 0), 100)
 
         if self.process is None or self.process.returncode != 0:
             _pretty_stderr = "\n".join(stderr)
